define("ace/ext/spellcheck",["require","exports","module"],function(e,t,n){text.spellcheck=!0,host.on("nativecontextmenu",function(e){if(host.selection.isEmpty()){var t=host.getCursorPosition(),n=host.session.getWordRange(t.row,t.column),s=host.session.getTextRange(n);host.session.tokenRe.lastIndex=0,host.session.tokenRe.test(s)&&(e=s+" "+PLACEHOLDER,text.value=e,text.setSelectionRange(s.length+1,s.length+1),text.setSelectionRange(0,0),inputHandler=function(t){if(t==e)return"";if(t.lastIndexOf(e)==t.length-e.length)return t.slice(0,-e.length);if(0==t.indexOf(e))return t.slice(e.length);if(t.slice(-2)==PLACEHOLDER){var s=t.slice(0,-2);if(" "==s.slice(-1))return s=s.slice(0,-1),host.session.replace(n,s),s}return t})}})});