module.exports = [require("./ACT.js"),require("./Adelaide.js"),require("./Brisbane.js"),require("./Broken_Hill.js"),require("./Canberra.js"),require("./Currie.js"),require("./Darwin.js"),require("./Eucla.js"),require("./Hobart.js"),require("./LHI.js"),require("./Lindeman.js"),require("./Lord_Howe.js"),require("./Melbourne.js"),require("./NSW.js"),require("./North.js"),require("./Perth.js"),require("./Queensland.js"),require("./South.js"),require("./Sydney.js"),require("./Tasmania.js"),require("./Victoria.js"),require("./West.js"),require("./Yancowinna.js")]
