'use strict';

// //////////////////////////////////////////////////////////////////////////////
// / DISCLAIMER
// /
// / Copyright 2014-2024 ArangoDB GmbH, Cologne, Germany
// / Copyright 2004-2014 triAGENS GmbH, Cologne, Germany
// /
// / Licensed under the Business Source License 1.1 (the "License");
// / you may not use this file except in compliance with the License.
// / You may obtain a copy of the License at
// /
// /     https://github.com/arangodb/arangodb/blob/devel/LICENSE
// /
// / Unless required by applicable law or agreed to in writing, software
// / distributed under the License is distributed on an "AS IS" BASIS,
// / WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// / See the License for the specific language governing permissions and
// / limitations under the License.
// /
// / Copyright holder is ArangoDB GmbH, Cologne, Germany
// /
// //////////////////////////////////////////////////////////////////////////////

const SwaggerContext = require('@arangodb/foxx/router/swagger-context');

module.exports =
  class Middleware extends SwaggerContext {
    constructor (path, fn) {
      if (!path) {
        path = '/*';
      } else if (path.charAt(path.length - 1) !== '*') {
        if (path.charAt(path.length - 1) !== '/') {
          path += '/';
        }
        path += '*';
      }
      super(path);
      this._middleware = fn;
      this._handler = fn;
      if (typeof fn.register === 'function') {
        this._handler = fn.register(this) || fn;
      }
    }

    _PRINT (ctx) {
      ctx.output += `[FoxxMiddleware: "${this.path}"]`;
    }
};
