module.exports = [require("./Amsterdam.js"),require("./Andorra.js"),require("./Astrakhan.js"),require("./Athens.js"),require("./Belfast.js"),require("./Belgrade.js"),require("./Berlin.js"),require("./Bratislava.js"),require("./Brussels.js"),require("./Bucharest.js"),require("./Budapest.js"),require("./Busingen.js"),require("./Chisinau.js"),require("./Copenhagen.js"),require("./Dublin.js"),require("./Gibraltar.js"),require("./Guernsey.js"),require("./Helsinki.js"),require("./Isle_of_Man.js"),require("./Istanbul.js"),require("./Jersey.js"),require("./Kaliningrad.js"),require("./Kiev.js"),require("./Kirov.js"),require("./Lisbon.js"),require("./Ljubljana.js"),require("./London.js"),require("./Luxembourg.js"),require("./Madrid.js"),require("./Malta.js"),require("./Mariehamn.js"),require("./Minsk.js"),require("./Monaco.js"),require("./Moscow.js"),require("./Nicosia.js"),require("./Oslo.js"),require("./Paris.js"),require("./Podgorica.js"),require("./Prague.js"),require("./Riga.js"),require("./Rome.js"),require("./Samara.js"),require("./San_Marino.js"),require("./Sarajevo.js"),require("./Saratov.js"),require("./Simferopol.js"),require("./Skopje.js"),require("./Sofia.js"),require("./Stockholm.js"),require("./Tallinn.js"),require("./Tirane.js"),require("./Tiraspol.js"),require("./Ulyanovsk.js"),require("./Uzhgorod.js"),require("./Vaduz.js"),require("./Vatican.js"),require("./Vienna.js"),require("./Vilnius.js"),require("./Volgograd.js"),require("./Warsaw.js"),require("./Zagreb.js"),require("./Zaporozhye.js"),require("./Zurich.js")]
