module.exports = [require("./Apia.js"),require("./Auckland.js"),require("./Bougainville.js"),require("./Chatham.js"),require("./Chuuk.js"),require("./Easter.js"),require("./Efate.js"),require("./Enderbury.js"),require("./Fakaofo.js"),require("./Fiji.js"),require("./Funafuti.js"),require("./Galapagos.js"),require("./Gambier.js"),require("./Guadalcanal.js"),require("./Guam.js"),require("./Honolulu.js"),require("./Johnston.js"),require("./Kiritimati.js"),require("./Kosrae.js"),require("./Kwajalein.js"),require("./Majuro.js"),require("./Marquesas.js"),require("./Midway.js"),require("./Nauru.js"),require("./Niue.js"),require("./Norfolk.js"),require("./Noumea.js"),require("./Pago_Pago.js"),require("./Palau.js"),require("./Pitcairn.js"),require("./Pohnpei.js"),require("./Ponape.js"),require("./Port_Moresby.js"),require("./Rarotonga.js"),require("./Saipan.js"),require("./Samoa.js"),require("./Tahiti.js"),require("./Tarawa.js"),require("./Tongatapu.js"),require("./Truk.js"),require("./Wake.js"),require("./Wallis.js"),require("./Yap.js")]
