module.exports = [require("./Azores.js"),require("./Bermuda.js"),require("./Canary.js"),require("./Cape_Verde.js"),require("./Faeroe.js"),require("./Faroe.js"),require("./Jan_Mayen.js"),require("./Madeira.js"),require("./Reykjavik.js"),require("./South_Georgia.js"),require("./St_Helena.js"),require("./Stanley.js")]
